; PRO:     chara_config
;
; Purpose: To calculate UV vectors for CHARA
;
; Input  : None
; Usage  : chara_config, baseline_s=baseline_s, baseline_arr=baseline_arr
;
; Results: baseline_s,   a structure of 30 baseline coordinates in [x, y, z] 
;                        in METERS                            (East, North, Up)
;          baseline_arr, an 3*30 array that include all baselines in [x, y,z],
;                        i.e., [ 01 01 01
;                                02 02 02       
;                                ... ... ]
;                        can be used in uvtrack.pro
;          baseline_arr2, an (3,6,6) array that includes all baselines as [*,0,1]
;									  [*,0,2]
;								... ...   [*,5,4]
;          baseline_2  , a structure contains baseline_arr2 and corresponding names.
;
;          All of the above 4 variables contain the same information.
;
; Note   : Code used for telescopes:    0   1   2   3   4   5
;                                       S1  S2  E1  E2  W1  W2
;          The conjugate of each baseline is also inculded, thus there are 30 
;              baseline vectors in total (15*2) .
;          The units are converted to Meters from Microns
;
; MZ, 01/29/05
;     05/19/06, find bug and fixed.
;               Note: changed on 05/19/06, baseline_arr1 was originally baseline_arr
;

Pro chara_config, baseline_s=baseline, baseline_arr1=baseline_arr, baseline_2=baseline2,$
                  baseline_arr2=baseline_arr2


 ;-- Telescope Coordinates

S1 = [        0.0d,           0.0d,              0.0d] /1d6 ;0
S2 = [-5748393.059d,   33581340.790d,     643880.188d] /1d6 ;1

E1 = [125337986.867d,  305925736.557d,  -5929522.324d] /1d6 ;2
E2 = [ 70414616.496d,  269731313.642d,  -2810776.414d] /1d6 ;3

W1 = [-175073611.569d, 216338950.344d, -10806621.168d] /1d6 ;4
W2 = [ -69080651.822d, 199355622.792d,    447470.433d] /1d6 ;5



 ; -- UV vectors

S1S2 = S2 - S1        ; 01
S1E1 = E1 - S1        ; 02
S1E2 = E2 - S1        ; 03
S1W1 = W1 - S1        ; 04
S1W2 = W2 - S1        ; 05

S2S1 = -S1S2          ; 10
S2E1 = E1 - S2        ; 12 
S2E2 = E2 - S2        ; 13
S2W1 = W1 - S2        ; 14
S2W2 = W2 - S2        ; 15

E1S1 = -S1E1          ; 20
E1S2 = -S2E1          ; 21
E1E2 = E2 - E1        ; 23
E1W1 = W1 - E1        ; 24
E1W2 = W2 - E1        ; 25

E2S1 = -S1E2          ; 30
E2S2 = -S2E2          ; 31
E2E1 = -E1E2          ; 
E2W1 = W1 - E2
E2W2 = W2 - E2

W1S1 = -S1W1
W1S2 = -S2W1
W1E1 = -E1W1
W1E2 = -E2W1
W1W2 = W2 - W1

W2S1 = -S1W2
W2S2 = -S2W2
W2E1 = -E1W2
W2E2 = -E2W2
W2W1 = -W1W2


baseline  = {S1S2:S1S2, S1E1:S1E1, S1E2:S1E2, S1W1:S1W1, S1W2:S1W2, $
             S2S1:S2S1, S2E1:S2E1, S2E2:S2E2, S2W1:S2W1, S2W2:S2W2, $
             E1S1:E1S1, E1S2:E1S2, E1E2:E1E2, E1W1:E1W1, E1W2:E1W2, $
             E2S1:E2S1, E2S2:E2S2, E2E1:E2E1, E2W1:E2W1, E2W2:E2W2, $
             W1S1:W1S1, W1S2:W1S2, W1E1:W1E1, W1E2:W1E2, W1W2:W1W2, $
             W2S1:W2S1, W2S2:W2S2, W2E1:W2E1, W2E2:W2E2, W2W1:W2W1 }

baseline_arr = [ [S1S2], [S1E1], [S1E2], [S1W1], [S1W2], $
             [S2S1], [S2E1], [S2E2], [S2W1], [S2W2], $
             [E1S1], [E1S2], [E1E2], [E1W1], [E1W2], $
             [E2S1], [E2S2], [E2E1], [E2W1], [E2W2], $
             [W1S1], [W1S2], [W1E1], [W1E2], [W1W2], $
             [W2S1], [W2S2], [W2E1], [W2E2], [W2W1] ]


baseline_arr2 = [ [ [0d,0d,0d], [S2S1], [E1S1], [E2S1], [W1S1], [W2S1] ], $
		  [ [S1S2], [0d,0d,0d], [E1S2], [E2S2], [W1S2], [W2S2] ], $
		  [ [S1E1], [S2E1], [0d,0d,0d],	[E2E1], [W1E1], [W2E1] ], $
		  [ [S1E2], [S2E2], [E1E2], [0d,0d,0d], [W1E2], [W2E2] ], $
		  [ [S1W1], [S2W1], [E1W1], [E2W1], [0d,0d,0d], [W2W1] ], $
		  [ [S1W2], [S2W2], [E1W2], [E2W2], [W1W2], [0d,0d,0d] ]    ]  	



name =          [ [ '  NA', 'S2S1', 'E1S1', 'E2S1', 'W1S1', 'W2S1' ], $
                  [ 'S1S2', 'NA  ', 'E1S2', 'E2S2', 'W1S2', 'W2S2' ], $
                  [ 'S1E1', 'S2E1', 'NA  ', 'E2E1', 'W1E1', 'W2E1' ], $
                  [ 'S1E2', 'S2E2', 'E1E2', 'NA  ', 'W1E2', 'W2E2' ], $
                  [ 'S1W1', 'S2W1', 'E1W1', 'E2W1', 'NA  ', 'W2W1' ], $
                  [ 'S1W2', 'S2W2', 'E1W2', 'E2W2', 'W1W2', 'NA  ' ]    ]

baseline2 = {baseline:baseline_arr2, name:name}


end


