; To predict visibilities for CHARA baselines for a given star.
; Usage:  chara_telescope_sel2,tnumber=4, baseline_cut=330,sour='Test',ra=5.,dec=1.,/proj, $
;                             ha=ha, hour_range=findgen(100)/50, diam=0.7, /hband [,/nops]
;
; MZ, 06/19/2007  Modified for MICR plan GUI 

pro baselength, tele=tele, baselength=baselength, basename  =basename

;Input: tele=[1,3,4,5]
;
;Output: baselength=baselength, basename  =basename

tele_id = ['S1',  'S2',  'E1',  'E2',  'W1',  'W2']
;             0     1       2      3     4      5

num= n_elements(tele)
tname = tele_id[tele]

chara_config, baseline_2=baseline

 ; make baseline pairs
 baselength = 0
 basename=' '
 tmp= sqrt(total((baseline.baseline)^2, 1))


for lp1=0, num-2 do begin
   For lp2 = lp1+1, num-1 do begin

    t1 = tname[lp1]
    t2 = tname[lp2]

    bname =t1+t2
    idx = where(bname eq baseline.name, cnt)
    if cnt ge 1 then begin
       tmp_length= tmp[idx]
         
       baselength=[baselength, tmp_length]
       basename  =[basename, bname]
    endif else message, 'baseline name error !'
   Endfor
endfor

baselength=baselength[1:*]
basename  =basename[1:*]

end



;---------------------------------------------------

pro projectedbaselength, ra=ra,dec=dec, ha=ha, $
    tele=tele, baselength=baselength, basename=basename

;Input: tele=[1,3,4,5]
;
;Output: baselength=baselength, basename  =basename
;
;


tele_id = ['S1',  'S2',  'E1',  'E2',  'W1',  'W2']
;             0     1       2      3     4      5

num= n_elements(tele)
tname = tele_id[tele]

chara_uv, ra=ra,dec=dec, ha=ha, uv=uv

 ; make baseline pairs
 baselength = 0
 basename=' '
 tmp= sqrt(uv.u^2 + uv.v^2 )

for lp1=0, num-2 do begin
   For lp2 = lp1+1, num-1 do begin

    t1 = tname[lp1]
    t2 = tname[lp2]

    bname =t1+t2
    idx = where(bname eq uv.name, cnt)
    if cnt ge 1 then begin
       tmp_length= tmp[idx]

       baselength=[baselength, tmp_length]
       basename  =[basename, bname]
    endif else message, 'baseline name error !'
   Endfor
endfor

baselength=baselength[1:*]
basename  =basename[1:*]

end

;----------------------------------------------------------



pro chara_telescope_sel2, tele=tele, baseline_cut=bcut, source=source,$
    ra=ra,dec=dec, ha=ha, projected=proj, $;set these if want projected baseline
    hour_range= haran, nops=nops, diam=diam0, band=band, fname=fname, $
    gui_text_id = g_text_id, gui_draw_id = g_draw_id, pinfo = info

; Usage:  chara_telescope_sel,tnumber=4, baseline_cut=330,sour='Test',ra=5.,dec=1.,/proj, $
;                           ha=ha,  hour_range=findgen(100)/50, diam=0.7,/hband, psid=psid

; chara_telescope_sel,sour='Deneb',ra=pso.ra,dec=pso.dec,/proj, $
;                     hour_range=findgen(101)/50-1., diam=2.460,/kband
;
;

; set /proj to make the baselines projected according to the source


If (band) eq 0 then begin 
wavelength=1.65d-6
bandname='H-Band'
endif else $
If band eq 1 then begin
wavelength=2.2d-6
bandname='K-Band'
endif

widget_control, g_text_id, /append, set_value=['Working at '+bandname+'       Wavelength: '+strtrim(string(wavelength),2)]



tnum = n_elements(tele)

If n_elements(diam0) eq 0 then widget_control, g_text_id, /append, set_value=['Please input the source diameter in MAS']
diam = diam0/1000.0/3600.0/180.0*!dpi   ; Convert MAS to Radian

If keyword_set(nops) eq 0 then begin
	set_plot,'ps'
	if n_elements(fname) eq 0 then fname=bandname+source+'_UVcoverageNbaseline.ps'
	device, filename=fname
	widget_control, g_text_id, /append, set_value=['-> Output to file :'+(*info).values.output_tab2]
	goodplot
	device, ysize=13, xsize=28, yoffset=0
	!P.multi=[0,2,1]
endif else begin
	goodplot
	!P.multi=[0,2,1]
endelse

if n_elements(bcut) eq 0 then bcut=330.0
tele_id = ['S1',  'S2',  'E1',  'E2',  'W1',  'W2']

widget_control, g_draw_id,  get_value=winid

case tnum of
       4: begin
           
          num=15

          baselen=-1.0
          basename = 'NA'
          telenames = ['NA','NA','NA','NA']
          count = -1

          chara_uv, ra=ra,dec=dec, ha=haran, uv=charauv     
 
          tname = tele_id[tele]         

          if keyword_set(proj) then begin

            If n_elements(ha) eq 0 then begin
               ha =0.
               widget_control, g_text_id, /append, set_value=['Default Hour Angle = 0']
            endif ;else print, 'Hour Angel =', ha
            projectedbaselength, ra=ra,dec=dec, ha=ha, $
            tele=tele, baselength=blength, basename=bname
          endif else $
          baselength, tele=tele, baselength=blength, basename=bname 
  
          baselen=[baselen,[blength]]
          basename = [basename, [bname]]
          telenames = [telenames, [tname]]        

          idx = where(blength le bcut, cnt)

          count=[count, cnt]

   ; PLot UV coverage

 tname_tmp=['']
 for lp11=0,2 do begin
   for lp12=lp11+1,3 do begin         
 tname_tmp=[tname_tmp,tname[lp11]+tname[lp12]]
   endfor
 endfor
 tname_tmp = tname_tmp[1:*]

 uv_idx_tmp=0
 for lp13=0, n_elements(tname_tmp)-1 do $ 
 uv_idx_tmp=[uv_idx_tmp, where(charauv.name eq tname_tmp[lp13], cnt)]
 uv_idx=uv_idx_tmp[1:*]

 plotsym,0,/fill
 dimension=n_elements(uv_idx)*n_elements(haran)
 telename=tname[0]+tname[1]+tname[2]+tname[3]

 if keyword_set(nops) eq 1 then wset, winid

 plot, reform(charauv.u[uv_idx,*],dimension), reform(charauv.v[uv_idx,*],dimension), $
   tit='UV Coverage of '+telename+' for '+source, xtit='u (meters)', ytit='v (meters)',$
   psym=8, yran=[-350,350], xran=[-350,350]
 xyouts,0,-365,'HA range: ['+strmid(strtrim(string(round(min(haran)*10))/10.,1),0,4)+', '+strmid(strtrim(string(round(max(haran)*10.))/10.,1),0,4)+']'
 oplot, -reform(charauv.u[uv_idx,*],dimension), -reform(charauv.v[uv_idx,*],dimension),psym=8

   ; PLot Visibility^2 curve for Uniform Disk approx.

 Bproj=findgen(340)
 x=!dpi*diam*Bproj/ wavelength
 v2_model=(2.0*beselj(x,1,/double)/x)^2.0


 x2 = !dpi*diam*blength/ wavelength
 v2_predict=(2.0*beselj(x2,1,/double)/x2)^2.0

 plot, Bproj, v2_model, xtit='Projected Baseline (meters)', $
       Ytit='Vis2', tit='UD Model for '+source+' @ '+bandname, yr=[0, max(v2_predict)]
 xyouts, 263,0.9, 'HA = '+strmid(strtrim(string((ha)),1),0,4)
 xyouts, 250,0.84, 'Diam= '+strmid(strtrim(string(max(diam0)),1),0,4)+' mas'


 oplot, blength, v2_predict, psym=8
 

          baselen=baselen[1:*]
          basename = basename[1:*]
          telenames = telenames[1:*]
          count = count[1:*]
  
          End    ; case of 4
     3: begin
        end
     2: begin
        end
     5: begin
        end
     6:begin
       end

endcase                  

if keyword_set(nops) eq 0 then begin
cleanplot,/silent
device, /close
set_plot,'x'
endif

;stop
end          



