; FUNCTION: chara_uv
;
; Propose: To calculate the uv coverage for chara, by using
;          uvtrack.pro and chra_config.pro
;
; Input  : RA, Dec : R.a & Dec of the source, in Radians
;          ha or mjd : Time of observation. Use either one
;                      HA, in HOURS
;                      MJD, in days (Modified Julian date)
;
; Return : ha, mjd : HA will be calculated if mjd is used
;          uv      : The uv coverage of CHARA from given time series
;                    uv={ u:u, v:v, num:num, name:name}
;                    in METERS                    
;
;              where num is the index of each beasline, made for the use 
;                           of tracking baselines.  
;                    name: name of baselines
;                    Each num and name refer to a paticular baseline.      
;                    e.g. 01 and S1S2 refer to u[0,*]& v[0,*]
;
;
; Note   : Code used for telescopes:    0   1   2   3   4   5
;                                       S1  S2  E1  E2  W1  W2
;          The conjugate of the baselines are also included, 
;          therefore, there are 30 baselines in total.  
;
; Remember to test output ha and mjd!
;
; Ming Zhao, 02/16/05
;            05/19/06, modified
; Ming Zhao, 10/13/06, applied the latest CHARA telescope position 


Pro chara_uv, ra=ra,dec=dec, ha=ha, mjd=mjd, uv=uv

;lon = -118.0566667d                        ; Degree
;lat = 34.226667d                           ; Degree

; New S1 Lon & Lat from telescope.chara file got on 10/13/06
 
LON =    -(118d0 + 3d0 / 60. + 25.31272d0 /3600.)
LAT =      34d0 + 13d0 /60. + 27.78130d0 /3600.

chara_config, baseline_arr1=b_arr


If n_elements(mjd) ne 0 then $
tmp = uvtrack(b_arr, ra,dec,lat,lon,mjd=mjd ) $
else if n_elements(ha) ne 0 then $
tmp = uvtrack(b_arr, ra,dec,lat,lon,ha=ha)

ha  = tmp.ha
mjd = tmp.mjd
;uv  = {u:tmp.u, v:tmp.v}  


 ; define index for uv tracks. 

uv = {  u:tmp.u, v:tmp.v, w:tmp.w, $
        num:['01', '02', '03', '04', '05', $
             '10', '12', '13', '14', '15', $
             '20', '21', '23', '24', '25', $
             '30', '31', '32', '34', '35', $
             '40', '41', '42', '43', '45', $
             '50', '51', '52', '53', '54']  , $
        name:['S1S2', 'S1E1', 'S1E2', 'S1W1', 'S1W2', $
              'S2S1', 'S2E1', 'S2E2', 'S2W1', 'S2W2', $
              'E1S1', 'E1S2', 'E1E2', 'E1W1', 'E1W2', $
              'E2S1', 'E2S2', 'E2E1', 'E2W1', 'E2W2', $
              'W1S1', 'W1S2', 'W1E1', 'W1E2', 'W1W2', $
              'W2S1', 'W2S2', 'W2E1', 'W2E2', 'W2W1' ]}

end
