
; PRO : obj_pos_mplan.pro
;
; Purpose: To calculate the HA of given sources (read-in using read_src_chara_plan2.pro)
;           for a given date at CHARA (or VLTI). And select the sources observable on that day from 
;           the given source list.
;          This is a good plot for choosing sources for obs.       
;          The daylight saving time will be used automatically!!
;
;          
; Input: day=day, month=month, year=year, /chara, [/vlti], [time=time], [/dops]
;        [file=fname, lon=lon, lat=lat,Tzone=tzone ;if not using default. ]
;
;        Time: default is 00:00, optional
;        If /chara or /vlti is set, then their lon, lat, Tzone will be used accordingly. 
;
; Output: a ps plot generated by the program
;
;
; NOTE : This pro calls CT2LST, JDCNV in the IDL astronomy library.
;
; Ming Zhao, 03/17/06
;            09/15/06, add R.A. at the bottom of the plot
;            06/10/07, adopted from obj_pos3. Modified for Mplan GUI
;

pro obj_pos_mplan, time=time, day=day, month=month, year=year, $
             dops=dops,  $
             file=fname, outfile=filename, lon=lon, lat=lat,Tzone=tzone ; Optional, if not using default

If n_elements(time) eq 0 then begin 
     time =0.0    ; default is 0am
     print, '--> Using default time: 00:00:00'
endif


if n_elements(lon) eq 0 then begin


     ;default lon, lat for CHARA

     lon = -118.0566667d                        ; Degree
     lat = 34.226667d                           ; Degree
     Tzone = 8d ; PST
     Print, '--> CHARA position: ', lon, lat 

     If (month ge 4) and (month le 10) then begin
     Tzone =7d ;PDT 
     Print, '--> Using Daylight Saving Time. Tzone: ', Tzone
     Endif
Endif


case month of
     	1:month_name='Jan'
     	2:month_name='Feb'
     	3:month_name='Mar'
	4:month_name='Apr'
	5:month_name='May'
	6:month_name='Jun'
	7:month_name='Jul'
	8:month_name='Aug'
	9:month_name='Sep'
	10:month_name='Oct'
	11:month_name='Nov'
	12:month_name='Dec'
endcase

date = month_name+' '+strtrim(string(day),1)+'_'+strtrim(string(year),1)
date2=strtrim(string(year),1)+month_name+strtrim(string(day),1)

CT2LST, Lst, Lon, Tzone, Time, Day, Month, Year  ; get LST in hours

data=read_src_chara_plan3(fname=fname)

src_ra = data.pos[0,*]
src_dec= data.pos[1,*]
   type= data.type


src_ha = lst- src_ra 
  idx = where(src_ha lt -12.0, cnt)
  if cnt ge 1 then src_ha[idx]=src_ha[idx]+24.0
 
  idx = where(src_ha gt 12.0, cnt)
  if cnt ge 1 then src_ha[idx]=src_ha[idx]-24.0


yran=[floor(min(src_dec)/10)*10-5 , ceil(max(src_dec)/10)*10 ]
xran=[-12., 12.]

;idx_xr = where(xran lt 0., cnt)
;if cnt ge 1 then xran(idx_xr) += 24.
;idx_xr = where(xran ge 24., cnt)
;if cnt ge 1 then xran(idx_xr) -= 24.


ticks = 5
tickv = [lst-10d0, lst-6d0, lst-2d0, lst+2d0, lst+6d0, lst+10d0]
idx_xr = where(tickv lt 0., cnt)
if cnt ge 1 then tickv(idx_xr) += 24.
idx_xr = where(tickv ge 24., cnt)
if cnt ge 1 then tickv(idx_xr) -= 24.


;stop


; --- Make Trial Plot ---

  plotsym, 0, 0.7, /fill
xran=[-7,7]
  plot, src_ha, src_dec, psym=8, xtit='Hour Angle (hrs)', ytit='Dec. (degree)', $
        tit=titl, xr=xran, xsty=4;
 xran2 = [lst+7., lst-7.]
 axis, xaxis=0, xran=xran2, xtit='R.A. (Hrs)',xsty=1,  xtick_get=xtv

idx_xr = where(xtv lt 0., cnt)
if cnt ge 1 then xtv(idx_xr) += 24.
idx_xr = where(xtv ge 24., cnt)
if cnt ge 1 then xtv(idx_xr) -= 24.

xtv =strmid(strtrim(string(round(xtv*10)/10.),1),0,4)




;====== plot and write to file if /dops ====

if keyword_set(dops) then begin
  dname = !d.name
  set_plot,'ps'
endif
goodplot

setcolors, NAMES=cnames, VALUES=cindx
ind1 = where(type eq 'ROT', cnt1)
  if cnt1 ge 1 then cl1 = getcolor('blue',cnames,cindx)
ind2 = where(type eq 'Be', cnt2)
  if cnt2 ge 1 then cl2 = getcolor('orange',cnames,cindx)
ind3 = where(type eq 'Bin', cnt3)
  if cnt3 ge 1 then cl3 = getcolor('red',cnames,cindx)
ind4 = where(type eq 'CAL', cnt4)
  if cnt4 ge 1 then cl4 = getcolor('forest',cnames,cindx)
ind5 = where(type eq 'EXO',cnt5)
  if cnt5 ge 1 then cl5 = getcolor('purple',cnames,cindx)

  ;--------------
    if time ne 0.0 then titl= 'Source Position at CHARA on '+date2+' Night - Local Time:'+string(time, format='(f3.1)') $ 
    else titl= 'Source Position at CHARA on '+date2+' Midnight'
    if keyword_set(dops) then begin
        ;filename='CHARA_src_pos_'+date2+'.ps'
       device, filename=filename, /color, bits_per_pix=8
    endif     

  plotsym, 0, 0.7, /fill, thick=0.5

  plot, src_ha, src_dec, psym=8, xtit='Hour Angle (hrs)', ytit='Dec. (degree)', $
        xr=xran, xsty=4+1,  ymargin=[4,3], yran=[-40, 90],ysty=1;
 
  if cnt1 ge 1 then oplot, src_ha[ind1], src_dec[ind1], psym=8, color=cl1
  if cnt2 ge 1 then oplot, src_ha[ind2], src_dec[ind2], psym=8, color=cl2
  if cnt3 ge 1 then oplot, src_ha[ind3], src_dec[ind3], psym=8, color=cl3
  if cnt4 ge 1 then oplot, src_ha[ind4], src_dec[ind4], psym=8, color=cl4
  if cnt5 ge 1 then oplot, src_ha[ind5], src_dec[ind5], psym=8, color=cl5


  xyouts, -4, 84, titl
  axis, xaxis=1, xran=xran, xtit='Hour Angle (Hrs)',xsty=1
  axis, xaxis=0, xran=xran2, xtit='R.A. (Hrs)',xsty=1,  xtickn=xtv;, subtit=tit
  oplot, [-6, -6], [-50,90], linesty=2
  oplot, [6, 6], [-50,90], linesty=2
  oplot, [0,0], [-50, 90], linesty=1

     if keyword_set(chara) then $
      oplot, [0,0], [lat, lat], psym=1, symsize=1.5


    idx=where((src_ha gt -7) and (src_ha) lt 5.8)
    if idx[0] ne -1 then xyouts, src_ha[idx]+0.15 , src_dec[idx], (data.name)[idx], charsize=0.8

 if keyword_set(dops) then begin 
  device,/close
  set_plot,dname
  wdelete
;  spawn, 'ps2pdf '+filename
;  spawn, 'rm -f 
 endif



 ;=========================================================
goto, skip
idx = where((src_ha ge -6.0) and (src_ha le 6.0), cnt)
if cnt ge 1 then begin  ; print to file
  src_ra = src_ra[idx]
  src_dec= src_dec[idx]
  name = data.name[idx]
  vmag = data.vmag[idx]
  hmag = data.hmag[idx]
  kmag = data.kmag[idx]
  otype= data.type[idx]
  stype= data.spec_type[idx]
  comm = data.comm[idx]
 
  idx2 = sort(hmag)

  src_ra = src_ra[idx2]
  src_dec= src_dec[idx2]
  name = name[idx2]
  vmag = vmag[idx2]
  hmag = hmag[idx2]
  kmag = kmag[idx2]
  type = type[idx2]
  comm = comm[idx2]
  stype= stype[idx2]

openw, lun, 'CHARA_src_sel_'+date2+'.txt',/get_lun

printf, lun, 'Available Sources for CHARA on '+date 
printf,lun,''
printf, lun, '   Source     Ra        Dec      V       H       K   Spec_type   Type  Comments'

printf, lun,'-----------------------------------------------------------------------------------------------------------------'
for lp=0, n_elements(name)-1 do begin
    printf, lun, name[lp], src_ra[lp], src_dec[lp], vmag[lp], hmag[lp],kmag[lp], stype[lp],  type[lp], comm[lp], $
            format='(A10, 1x, f7.3,2x, f8.3, 2x, f6.2,2x, f6.2,2x,f6.2,2x, A7, 3x, A4, 3x, A)'
endfor

free_lun,lun
close,/all

Endif

skip:

end

