
PRO obs_time2,  ra=ra, dec=dec, tele=tele, pop_num=pop_num, starttime=startt, endtime=endtime, $
		date=day, mon=mon, year=year,daylightsaving=daylightsaving, obst= obst, $
                ha_ran=ha_ran, cvtime=cvtime, cv_sunset=cv_sunset, cv_sunrise=cv_sunrise, $
		do_cvtime=do_cvtime, path=path, poprange=range

; Purpose : To calculate the Hour Angle range for each source for given DEC, 
;           given telescops and pops.
;           Dec. can be an array. To be used to make observation plan for CHARA.
;		
;
; Input   : dec, the DEC. of one or several sources, in DEGREE.
;	    ra , in HOURS
;           tele, the index of telescopes used.
;                 S1  S2  E1  E2  W1  W2
;                  0  1   2   3   4   5
;           pop_num, the pop number for each telescope. 
;		     There are 5 pops for each tele. from 0, 4
;           day, mon, year, the date of observation that used to get civil time.
; 
;	    starttime : Observation starts STARTTIME hour after sunset, default = 1.0 hour
;
;	    endtime   : observation ends ENDTIME hour before sunrise, defualt = 0. hour
;          
;           obst : the obs. time shreshold, default =0.5 hour
;
;	    /daylightsaving, set this key word while using day light saving time!
;	    /do_cvtime, set this keyword to enable calculating local civil time.
;
;           Path :  the path of archival data
;
; Output  : ha_ran, the range of hour angles of the sources that we can observe at
;		    given date for given telescopes, given pop configuration
;           ha_ran = [min1, min2, max1, max2] for 2 slots. 
;               or = [min, max, 0,0] when there is only one ha_ran slot 
;                                                       
;           cvtime, then civil time of the hour angle range for given date.
;	    cv_sunset, cv_sunrise , local time at sunrise and sunset
;
; Note    : the default longitude and timezone are for CHARA
;
; MZ, 08/11/05
; 11/16/05, modified the HA range searching part
;           Fixed the bug that happens when there is a gap in HA range which separates the range 
;              to 2 parts.
;           Tested for 2 teles --> Good
; 05/18/06, added path for archival data
;

If n_elements(startt) eq 0 then startt = 1.0d0    ; start time after sunset
If n_elements(endtime) eq 0 then endtime = 0.0	      ; end time before sunrise 
If n_elements(obst) eq 0 then obst =0.5
If n_elements(path) eq 0 then message,'obs_time: Please give the path of tha archival data!'

  

  ; Calculate HA of the Sun at sunset

jdcnv, year,mon,day,0, julianD 
sunpos, julianD, ra_sun, dec_sun, /radian
dec_sun = dec_sun/!dpi * 180d 
ra_sun = ra_sun / !dpi *12d
sun_ha_ran = tele_limit(DEC=dec_sun, phi= 0d)
sun_set = sun_ha_ran[1]
sun_rise= sun_ha_ran[0]

delta_ra = ra - ra_sun
        ha_at_sset  = sun_set - delta_ra + startt       ; Add STARTTIME to get the REAL observation start time! 

                ha_idx = where( ha_at_sset lt -12d)
                        if ha_idx[0] ne -1  then ha_at_sset[ha_idx] = ha_at_sset[ha_idx] + 24d
                ha_idx = where( ha_at_sset gt 12d)
                        if ha_idx[0] ne -1 then ha_at_sset[ha_idx] = ha_at_sset[ha_idx] - 24d

        ha_at_srise = sun_rise - delta_ra - endtime	   ; Add ENDTIME so that observation ends before sunrise!

                 ha_idx = where( ha_at_srise lt -12d)
                        if ha_idx[0] ne -1 then ha_at_srise[ha_idx] = ha_at_srise[ha_idx] + 24d
                 ha_idx = where( ha_at_srise lt ha_at_sset)     ; make sure sunrise time is larger than sunset
	                if ha_idx[0] ne -1 then ha_at_srise[ha_idx] = ha_at_srise[ha_idx] + 24d
        dha_sun = ha_at_srise - ha_at_sset                      ; this should always positive number
        If where(dha_sun lt 0.) ne -1  then message, 'sunrise time should be larger than sunset time! (obe_time.pro)'   

 ; -------------------------------------

dim = size(range, /dimensions)
ha_ran = dblarr(4, n_elements(ra))      ; ha_ran =  [t1,  t2,        t3,  t4 ]
                                        ;            1st slot        2nd slot
                                        ; Sometimes the range has two separated slots

dec_idx = round((dec+40.)* 6.)   ; 40 is the Dec indx offset of RANGE

ha_ran = float(range[*, dec_idx])/20. -6.   ; convert from pix to HA

dha_ran = ha_ran[1, *] - ha_ran[0, *] + ha_ran[3, *] - ha_ran[2, *]

For lp2 = 0, n_elements(dec)-1 do begin
  ; ------- Get the valid HA range for each sources  ----------------------
  ; There are several possible cases


     if dha_ran[lp2] eq 0. then begin
        ha_ran[*, lp2] = [0.,0.,0.,0.]
        continue
     endif

     If ( ha_ran[3, lp2] - ha_ran[2, lp2] ) eq 0. then begin       ; only one slot exist

        ; case 1 :

;        If (ha_ran[0,lp2] ge ha_at_sset ) and (ha_ran[1,lp2] ge ha_at_sset) and (ha_ran[0,lp2] le ha_at_srise) $
;           (ha_ran[1,lp2] le ha_at_srise) then do nothing 

   
        ; case 2 : 

	If  (ha_at_sset[lp2] gt ha_ran[0, lp2]) and (ha_at_sset[lp2] lt ha_ran[1, lp2]) then $
		ha_ran[0, lp2] = ha_at_sset[lp2]  
	If (ha_at_srise[lp2] gt  ha_ran[0, lp2]) and (ha_at_srise[lp2] lt ha_ran[1, lp2]) then $
		ha_ran[1, lp2] = ha_at_srise[lp2]

       ; case 3: 

	If ( (ha_at_sset[lp2] ge ha_ran[1, lp2]) and (ha_at_srise[lp2] ge ha_ran[1, lp2]) ) or $
           ( (ha_at_sset[lp2] le ha_ran[0, lp2]) and (ha_at_srise[lp2] le ha_ran[0, lp2]) ) then begin
                if (ha_at_srise[lp2] gt 12.) then tmp_ha = ha_ran[1, lp2]+24 else tmp_ha = ha_ran[1, lp2]
                if tmp_ha gt ha_at_srise[lp2] then ha_ran[1, lp2] = ha_at_srise[lp2]-24. else $
                if ((ha_at_sset[lp2] ge tmp_ha) and ((ha_at_srise[lp2] mod 24) ge tmp_ha)) then ha_ran[*, lp2] = [0,0,0,0] 
        Endif

       If ( (ha_at_sset[lp2] le ha_ran[0, lp2]) and (ha_at_srise[lp2] le ha_ran[0, lp2]) ) then  ha_ran[*, lp2] = [0,0,0,0]

       ; case 4: 

       if ha_ran[1, lp2] - ha_ran[0, lp2] le 0. then ha_ran[*, lp2] = [0,0,0,0]


     Endif else If ( ha_ran[3, lp2] - ha_ran[2, lp2] ) ge  obst  then begin   
                    ; when the 2nd slot is larger than the obs. shreshold then count it as real.

       ; case 1 :

;      If (ha_ran[2,lp2] ge ha_at_sset[lp2] ) and (ha_ran[3,lp2] ge ha_at_sset[lp2]) and (ha_ran[2,lp2] le ha_at_srise[lp2]) $
;           (ha_ran[3,lp2] le ha_at_srise[lp2]) then do nothing 


        ; case 2 : 

        If  (ha_at_sset[lp2] gt ha_ran[2, lp2]) and (ha_at_sset[lp2] lt ha_ran[3, lp2]) then $
                ha_ran[2, lp2] = ha_at_sset[lp2]
        If (ha_at_srise[lp2] gt  ha_ran[2, lp2]) and (ha_at_srise[lp2] lt ha_ran[3, lp2]) then $
                ha_ran[3, lp2] = ha_at_srise[lp2]


       ; case 3: 

        If ( (ha_at_sset[lp2] ge ha_ran[3, lp2]) and (ha_at_srise[lp2] ge ha_ran[3, lp2]) ) or $
           ( (ha_at_sset[lp2] le ha_ran[2, lp2]) and (ha_at_srise[lp2] le ha_ran[3, lp2]) ) then $
                ha_ran[2:3, lp2] = [0. , 0.]

      Endif

Endfor


  ; Convert Hour angle to Local time

if keyword_set(do_cvtime) eq 1 then begin 

lon =  -118.2237d		; default for Chara
if keyword_set(daylightsaving) eq 0 then timezone = 8. else timezone=7.

num_ra = n_elements(ra)
If num_ra gt 1 then ra = transpose(ra)
lst1 = ra + ha_ran[0,*]
lst2 = ra + ha_ran[1,*] 
lst3 = ra + ha_ran[2,*]
lst4 = ra + ha_ran[3,*]


lst = [lst1, lst2, lst3, lst4]
cvtime = dblarr(4, num_ra)

For lp3 = 0, num_ra-1 do begin

cvtime[0,lp3] = lst2ct(lst1[lp3], lon, timezone, day, mon, year)
cvtime[1,lp3] = lst2ct(lst2[lp3], lon, timezone, day, mon, year)
cvtime[2,lp3] = lst2ct(lst3[lp3], lon, timezone, day, mon, year)
cvtime[3,lp3] = lst2ct(lst4[lp3], lon, timezone, day, mon, year)

endfor


cvidx = where(abs(cvtime[0,*] - cvtime[1,*] ) lt obst)
if cvidx[0] ne -1 then cvtime[*, cvidx] = 0.

cvidx = where(abs(cvtime[3,*] - cvtime[2,*] ) lt obst)
if cvidx[0] ne -1 then cvtime[2:3, cvidx] = 0.


cv_sunset  = lst2ct(ra_sun + sun_set, lon, timezone, day, mon, year)
cv_sunrise = lst2ct(ra_sun + sun_rise, lon, timezone, day, mon, year)



Endif

end

