
; PRO plot_poprange
;
; Purpose: Given Telescope numbers, and pop numbers, plot the range of POP coverage
;          using archive range data, and compare with the source position chart
;          created by obj_pos.pro
;
;  Usage : plot_poprange, tele=[1,3,4,5], pop=[1,2,3,5]  [, /dops] [, filename=fname]
;
;          if no filename is given then use default and save plot to current directory 
;
; 09/25/06, mz
; 06/14/07, MZ  

pro plot_poprange2, tele=tele, pop=pnum0, dops=dops, filename=fname, gui_id=gid

;path = '/n/Virgo1/mingzhao/Research/chara/planning/range_data/'
;print, '-->PLOT_POPRANGE: Using default path   '+path 
path='poprange/'


num= n_elements(tele)
tmp = make_array(num, value=1)
pnum = pnum0 -tmp

telename = ['S1', 'S2', 'E1', 'E2', 'W1', 'W2']
popname  = ['0', '1', '2', '3', '4', '5']


tname=''
pname=''

for lp = 0, num-1 do begin
        tname = tname + telename[tele[lp]]
        pname = pname + popname[pnum[lp]]
endfor


print, '-->Working on Tele: '+tname+'  POP: '+ pname + ' (internal index)'
 ; In fact, the Range data used for the plot can be generated by running:
 ;  popcover, tele=tele, pop_num= pnum, range=range,  xr=ha, yr=decli 
 ; Since it is also saved in the archive, we load in the archive data as a fast approach.
 
restore, path+'pop_ha_range'+'_T'+tname+ '.idl'  ; resotre pop_range={flag:flag, range:range, popname:popname}

p_ind = where(pop_range.popname eq pname, pcnt)
if pcnt ne 1 then message, 'No Matching POP' else range=*(pop_range.range[p_ind[0]])

if pop_range.flag[p_ind[0]] eq 0b then begin
   if n_elements(gid) ne 0 then widget_control, gid, /append, set_value=['No coverage for this POP: '+strtrim(string(pname+1111),1)] $
   else print, '-->plot_poprange: No coverage for this POP: '+strtrim(string(pname+1111),1)
   goto, skip
endif

ha = findgen(280)/20.-7.
dec= findgen(780)/6.-40.


pname = strtrim(string(pname+1111),1)


if keyword_set(dops) then begin
        set_plot,'ps'
	goodplot
        if n_elements(fname) eq 0 then fname = 'coverage_T'+tname+'_P'+pname +'.ps'
        
 	device, filename=fname,/color
print, '-->PS output: '+fname
endif	

;if !d.name eq 'X' then loadct, 0 else loadct, 8
 loadct, 8

range2 = make_array(280, 780, value=160.)

  ; get the telescope limit
tele_limit = tele_limit(dec=(findgen(780)/6.-40), phi=20d)  ; 20degree is the lowest elevation allowed
tele_limit[where(tele_limit lt -6.)] = -12
tele_limit[where(tele_limit gt 6.)] = 12


limit_in_pix = round((tele_limit + 7. )*20)

  ; assign values 
for lp=0, 780-1 do begin
    if range[1, lp] - range[0,lp] ne 0b then range2[20+range[0,lp]:20+range[1,lp], lp] = 100
    if range[3, lp] - range[2,lp] ne 0b then range2[20+range[2,lp]:20+range[3,lp], lp] = 100 

      ; assign limit  
    if tele_limit[1, lp] - tele_limit[0, lp] ne 24. then begin 
        range2[0:limit_in_pix[0,lp], lp] = 0
        range2[limit_in_pix[1,lp]:*, lp] = 0
    endif
endfor
 
    range2[0:19, *] = 160
    range2[260:*, *] =160
range2[19, *] = 0.
range2[260, *] = 0.


  ; get the telescope limit
tele_limit = tele_limit(dec=(findgen(780)/6.-40), phi=20d)  ; 20degree is the lowest elevation allowed
 


range = range2

goodplot

if !d.name eq 'X' then $
   image_cont2, range[20:260, *], xv=ha[20:260], yv=dec, xt='Hour Angle (Hrs)', yt='Dec (Degrees)', tit='POP Coverage for Tele '+tname + '  POP: '+pname, ymargin=[4,3],/noco  else $
   image_cont2, range, xv=ha, yv=dec, xt='Hour Angle (Hrs)', yt='Dec (Degrees)', tit='POP Coverage for Tele '+tname + '  POP: '+pname, ymargin=[4,3],/noco


if keyword_set(dops) then device, /close
set_plot,'x'
cleanplot, /silent
range2=0  ; free mem.

skip:

end


