
PRO read_src_chara_plan, file=fname, src_pos=src, weight=wei, src_name=srcname, comment=comment, path=path

; Purpose  : Read the source list for chara planning, convert to radian. 
;
; input    : file, the filename of source list. 
;		   The format of the source list should be the same as 'src_test.txt' 
;			SOURCE        RA            Dec.       Weight  V      J      H     K       Comment
;                       -------------------------------------------------------------------------------------
;			Regulus   10 08 22.3107 +11 58 01.945  1.0    1.4   1.665  1.658  1.640    Elongated
;			Del Cep   22 29 10.2663 +58 24 54.715  1.0    4.1   2.865  2.608  2.354    Cepheid
;			Alp Cep   21 18 34.7715 +62 35 08.061  1.0    2.4   2.154  2.134  2.066    Elongated
;			Alp Oph   17 34 56.0705 +12 33 36.125  20.    2.1   1.833  1.718  1.683    Elongated
;			Iota Peg  22 07 00.6661 +25 20 42.402  20.0   3.8   2.954  2.729  2.564    Calibration Binary
;			... ...
;
;            Path, the path of the archive data, default='/n/Virgo1/mingzhao/Research/chara/planning/range_data/' 
;
; Output   : src_pos, [ra, dec] of sources, in Hours and Degree, respectivley
;	     src_name, name of all sources
;	     weight,  weight of observational priority for sources
;
; MZ,  08/05
;	09/10/05, last modified
;       05/17/06, last modified the default path of archival data

if n_elements(path) eq 0 then path = '/n/Virgo1/mingzhao/Research/chara/planning/range_data/'
restore, path+'source_template_chara.idl'		; restore TEMP from file
data = read_ascii(fname, template=temp)

srcname = data.field1

src_ra  = (data.field2 + data.field3/60. + data.field4/3600.) 	; in Hours

factor = fltarr(n_elements(data.field5))
ind = where(data.sign eq '+', cnt)
if cnt ge 1 then factor[ind] = 1.0                         ; Get the sign of Dec.

ind2 = where(data.sign eq '-', cnt)
if cnt ge 1 then factor[ind2] = -1.0

src_dec = factor*(data.field5 + data.field6 / 60. + data.field7 / 3600.) 	; in Degree
src	= dblarr(2, n_elements(src_ra))
src[0,*] = src_ra
src[1,*] = src_dec

wei	= data.field8
comment = data.field9

End

