

PRO src_plan2, src_list= src, weight= weight, optimize=optimize, obstime=obst, npop=npop, tele=tele, $
			date=day, mon=mon, year=year,daylightsaving=daylightsav, $
			sunset=cv_sunset, sunrise=cv_sunrise, starttime=starttime, endtime=endtime, $
			max_slot=max_slot, name_slot=name_slot, cvtime=cvtime, verbose=verbose, $
                        namefix=namefix, path=path, popfix=popfix0, gui_id=gid

; Purpose   :  To get the obs. time plan for given sources and GIVEN telescopes for CHARA
;		There are two ways to do this: 
;			1.  maximize the number of avaliable sources and then weighted obs. time of sources that we can observe.
;			2.  maximize the obs. time of sources according to their weights.
; 
; Input	    :  	src = [ ra (hours), dec (degrees) ],  list of source we want to observe.
;		
;		tele,  the telescopse to be used, e .g. tele=[0,1,4,5]  for S1 S2 W1 W2
;
;                       'S1', 'S2', 'E1', 'E2', 'W1', 'W2'
;                         0     1     2     3     4     5
;
;               POPFIX = e.g., [1, 2, 3,4] for 4_Tele
;                        Set this parameter to fix the choice of pops. '0' means no choice, 
;                        e.g., [0,0,0,3] means fix the last pop to be 3 and search for the other pops.
;                        Default = [0,0,0,0]
;
;
;  	       	Optimize  =  1 --> maximize the number of available sources and then weighted obs. time of observable sources.
;		  	  =  2 --> maximize the obs. time of sources based on their weights.
;
; 		obs_time: the observation time threshold for each avaliable source. 
;			  In hours. Default is 0.5 hour
;		npop: the number of pop configurations that will be taken after the selection.
;		      Default is 3, that is, for each telescope group, the best 3 pops that satisfy the criteria will be chosen.
; 
;		weight, the weight of each source used in optimization, default is 1 for each source.
; 
;		date, mon, year	:  The observation time.
;
;		starttime	:  Observation starts STARTTIME hours after sunset, default = 1.0 hour
;
;		endtime		:  Observation ends ENDTIME hours beforee sunrise, default = 0.0 hour.
;
;               path            :  data path for archival data.
;
;               namefix         :  the postfix for the output POPs. This psotfix will make sure the output is consistent with the
;                                  REAL POP names, instead of the idl version (i.e., start from 0).
;
;               /daylightsav
;		/verbose, set this keyword to get detail info.
;                
;
; Output    : 	sunset, sunrise : local sunset and sunrise time
;		max_slot	: gives the observable time duration of the sources.
;		cvtime		: gives the local observation time (start - end) of the sources,
;                                   If there are two slots, then will give [time1, time2,     time3, time4]
;                                                                            1st slot         2nd slot
;		name_slot       : names of the best POP setup
;
;
; MZ, 08/2005
; 11/15/05, modified
;
; 05/19/06, added path for archival data
; 06/16/2007, modified for GUI


if n_elements(path) eq 0 then message, 'src_plan: Please give path to archival data!'
telename = ['S1', 'S2', 'E1', 'E2', 'W1', 'W2' ]
;              0     1     2     3     4     5

popname  = ['0', '1', '2', '3', '4']

tname=''
for lp1 = 0, n_elements(tele)-1 do tname += telename[tele[lp1]]


If n_elements(popfix0) eq 0 then popfix =[-1,-1,-1,-1]    ; No fixed POPs
If n_elements(popfix0) ne 0 then popfix =popfix0- [1,1,1,1]


If n_elements(obst) eq 0 then obst = 0.5d     ; default obs. time is at least 0.5 hour for each source.
If n_elements(npop) eq 0 then npop = 3 	      ; use the best 3 pop config as default

If keyword_set(verbose) eq 0 then verbose =1   ; default is to verbose some detail
if keyword_set(verbose) eq 1 then begin
  If n_elements(gid) eq 0 then begin
  
    If keyword_set(daylightsav) eq 1 then print, '--> Using Day Light Saving time.'
    Print, '    -------------------------------'
    Print, format='("--> The observation time thredshold for each avaliable source is : ", f4.2, " hours")', obst
    Print, '--> You will get  '+strtrim(string(npop),1)+ '  best POP choices.'
    If optimize eq 1 then print, '--> Optimizing the number of available sources and weighted obs. time ...'
    if optimize eq 2 then begin 
	print, '--> Optimizing the obs. time for sources based on their weights...'
;	Print, '      Please give large weights to sources that have high priority, e.g. 10'
    endif

  endif else begin
    If keyword_set(daylightsav) eq 1 then $
    widget_control, gid, /append, set_value=['--> Using Day Light Saving time', '    -------------------------------',$
                                             "--> The observation time thredshold for each avaliable source is : "+strtrim(string(obst),2)+$
                                             " hours"]
  endelse


Endif    ; verbose


If n_elements(weight) eq 0 then weight = make_array(n_elements(src[0,*]), value=1.0)

n_src  = n_elements(src[0, *])
n_tele = n_elements(tele)
n_pop_config = 5^n_tele

pname = strarr(n_pop_config)

count = 0

data  = make_array(4, n_src, n_pop_config, /double)
slot  = make_array(n_src, n_pop_config, /double)
cnt2 = make_array( n_pop_config, /double)


fname = 'pop_ha_range_T'+tname
fname = path+fname+'.idl'
restore, fname    ; load pop_range={range:pop_ha_range, flag:flag, popnames:popnames}


If n_tele eq 4 then begin

   If popfix[0] eq -1 then begin
      i0=0
      i1=4
   endif else begin
      i0=popfix[0]
      i1=i0
   endelse

   If popfix[1] eq -1 then begin
      j0=0
      j1=4
   endif else begin
      j0=popfix[1]
      j1=j0
   endelse

   If popfix[2] eq -1 then begin
      k0=0
      k1=4
   endif else begin
      k0=popfix[2]
      k1=k0
   endelse

   If popfix[3] eq -1 then begin
      l0=0
      l1=4
   endif else begin
      l0=popfix[3]
      l1=l0
   endelse

;print, i0,i1, j0,j1, k0,k1,l0,l1

;stop
   For i = i0,i1 do begin 
	for j= j0, j1 do begin
		   for k =k0, k1 do begin
			for l=l0,l1 do begin
			  pname[count] = strtrim(string(i),1)+strtrim(string(j),1)+strtrim(string(k),1)+strtrim(string(l),1) 	
			  pop=[i,j,k,l]
                               
                             ; Filter invalid data sets in pop_range
                          p_ind = where(pop_range.popname eq pname[count], p_ind_cnt)
                          if p_ind_cnt eq 1 then begin
                             p_ind = p_ind[0]
                             if (pop_range.flag)[p_ind] eq 0b then continue else range=*(pop_range.range[p_ind])
                          endif else message, 'No matching POPS'
;stop

	  	obs_time2, ra=src[0,*], dec=src[1,*], poprange=range,  ha_ran=ha_ran, year=year, obst=obst, $
			 mon=mon, date=day, daylightsaving=daylightsav, starttime=starttime, endt=endtime, path=path

		data[*, *, count] = ha_ran
		slot[*, count] = abs(ha_ran[1,*] - ha_ran[0,*] + ha_ran[3,*] - ha_ran[2,*] )

		If optimize eq 1 then begin
		tmp = where( slot[*, count] ge obst)
		if tmp[0] ne -1 then tmp2 = n_elements(tmp) else tmp2 =0		
		cnt2[count] = tmp2
		endif


		If optimize eq 2 then begin 
			tmp = where( slot[*, count] ge obst)
	                if tmp[0] ne -1 then begin
				tmp2 = weight * slot[*,count] 
				tmp2 = total( tmp2[tmp] )
			endif else tmp2 =0
        	        cnt2[count] = tmp2
			
		endif
		
		count++
;stop
			endfor
		  endfor
	endfor
   Endfor

Endif 



If n_tele eq 3 then  begin
   For i = 0,4 do begin
        for j= 0, 4 do begin
                   for k =0, 4 do begin

                          pop=[i,j,k]
                          pname[count] = strtrim(string(i),1)+strtrim(string(j),1)+strtrim(string(k),1)


                             ; Filter invalid data sets in pop_range
                          p_ind = where(pop_range.popname eq pname[count], p_ind_cnt)
                          if p_ind_cnt eq 1 then begin
                             p_ind = p_ind[0]
                             if (pop_range.flag)[p_ind] eq 0b then continue else range=*(pop_range.range[p_ind])
                          endif else message, 'No matching POPS'

                obs_time2, ra=src[0,*], dec=src[1,*], poprange=range,  ha_ran=ha_ran, year=year, obst=obst, $
                         mon=mon, date=day, daylightsaving=daylightsav, starttime=starttime, endt=endtime, path=path


                data[*, *, count] = ha_ran
                slot[*, count] = abs(ha_ran[1,*] - ha_ran[0,*] +  ha_ran[3,*] - ha_ran[2,*] )

                If optimize eq 1 then begin
                tmp = where( slot[*, count] ge obst)
                if tmp[0] ne -1 then tmp2 = n_elements(tmp) else tmp2 =0
                cnt2[count] = tmp2
                endif



                If optimize eq 2 then begin
                        tmp = where( slot[*, count] ge obst)
                        if tmp[0] ne -1 then begin
                                tmp2 = weight * slot[*,count]
                                tmp2 = total( tmp2[tmp] )
                        endif else tmp2 =0
                        cnt2[count] = tmp2

                endif

		count++
                  endfor
        endfor
   Endfor
Endif



If n_tele eq 2 then  begin
   For i = 0,4 do begin
        for j= 0, 4 do begin

                pop=[i,j]
                pname[count] = strtrim(string(i),1)+strtrim(string(j),1)

                             ; Filter invalid data sets in pop_range
                          p_ind = where(pop_range.popname eq pname[count], p_ind_cnt)
                          if p_ind_cnt eq 1 then begin
                             p_ind = p_ind[0]
                             if (pop_range.flag)[p_ind] eq 0b then continue else range=*(pop_range.range[p_ind])
                          endif else message, 'No matching POPS'

                obs_time2, ra=src[0,*], dec=src[1,*], poprange=range,  ha_ran=ha_ran, year=year, obst=obst, $
                         mon=mon, date=day, daylightsaving=daylightsav, starttime=starttime, endt=endtime, path=path


                data[*, *, count] = ha_ran
                slot[*, count] = abs( ha_ran[1,*] - ha_ran[0,*] +  ha_ran[3,*] - ha_ran[2,*] )

                If optimize eq 1 then begin
                tmp = where( slot[*, count] ge obst)
                if tmp[0] ne -1 then tmp2 = n_elements(tmp) else tmp2 =0
                cnt2[count] = tmp2
                endif


                If optimize eq 2 then begin
                        tmp = where( slot[*, count] ge obst)
                        if tmp[0] ne -1 then begin
                                tmp2 = weight * slot[*,count]
                                tmp2 = total( tmp2[tmp] )
                        endif else tmp2 =0
                        cnt2[count] = tmp2

                endif

                count++

        endfor
   Endfor
Endif 


	; ----------------------------

max_slot  = dblarr(n_src, npop)
name_slot = strarr(npop)
slot_time = dblarr(4, n_src, npop)


For lp = 0, npop-1 do begin
			idx = where( cnt2 eq max(cnt2) )

                        if n_elements(idx) eq 1 then begin

                                max_slot[*,lp]  = slot(*, idx)
                                name_slot[lp] = pname(idx)
				slot_time[*, *,lp]= data[*,*,idx]

                                cnt2(idx) = 0
                        Endif else $
                        if n_elements(idx) gt 1 then begin
				If optimize eq 1 then begin
	                                max_arr= dblarr(n_elements(idx))
	                                for lp1 = 0, n_elements(idx)-1 do $     		; use 2nd criteria
	                                        max_arr(lp1) = total( slot[*,idx(lp1) ] * weight  )
	                                max_slot_idx = where(max_arr eq max(max_arr))
        	                        max_slot_idx = max_slot_idx[0]				; use the 1st element if still has redundance
                	                max_slot[*, lp]     = slot(*, idx(max_slot_idx))
                        	        name_slot[lp]    = pname(idx(max_slot_idx))
                                	slot_time[*, *,lp]= data[*,*,idx(max_slot_idx)]
				Endif

				If optimize eq 2 then begin
					max_arr= dblarr(n_elements(idx))
                                        for lp1 = 0, n_elements(idx)-1 do begin                     ; use 2nd criteria
						idx4 = where(weight eq max(weight) )
                                                max_arr(lp1) = slot(idx4[0], idx(lp1))
					endfor
                                        max_slot_idx = where(max_arr eq max(max_arr))
                                        max_slot_idx = max_slot_idx[0]          		; use the 1st element if still has redundance
                                        max_slot[*, lp]     = slot(*, idx(max_slot_idx))
                                        name_slot[lp]    = pname(idx(max_slot_idx))
                                        slot_time[*, *,lp]= data[*,*,idx(max_slot_idx)]
				Endif
		
				cnt2(idx(max_slot_idx)) = 0

                        endif
Endfor


  ; Calculate local time for selected time slots


cvtime = dblarr(4, n_src, npop)

For lp3 = 0, npop-1 do begin

pname = name_slot[lp3] 

 p_ind2 = where(pop_range.popname eq pname, p_ind_cnt)
                          if p_ind_cnt eq 1 then begin
                             p_ind2 = p_ind2[0]
                             range=*(pop_range.range[p_ind2])
                          endif else message, 'No matching POPS'



obs_time2,  ra=src[0,*], dec=src[1,*], poprange=range, path=path, obst=obst, $
                date=day, mon=mon, year=year,daylightsaving=daylightsav, starttime=starttime, endt=endtime, $
                ha_ran=ha_ran, cvtime=ct, /do_cvtime , cv_sunset=cv_sunset, cv_sunrise=cv_sunrise

;print, transpose(ha_ran[1,*]-ha_ran[0,*])
cvtime[*,*,lp3] = ct

Endfor

If keyword_set(verbose) eq 1 then begin
    if n_elements(gid) ne 0 then begin
        tnames=''
        for lp=0, n_elements(tele)-1 do tnames+=telename[tele[lp]]+' '
        slotnames=''
        for lp=0, n_elements(name_slot)-1 do slotnames+=strtrim(name_slot[lp]+ namefix , 2)+'  ' 
        widget_control, gid, /append, set_value=['---> Telescope: '+tnames,'---> The best POPs are: '+slotnames, '']
    endif else begin
	print, format='("--> Local sunset time : ", f4.1, " pm,  rises time : ",f4.1, " am")', cv_sunset, cv_sunrise
	print, format='("--> Observation starts ", f4.1, " hour after sunset, ends ", f4.1, " hour before sunrise")', Starttime,Endtime 
	print, '---> Telescope: ', telename[tele]
	print, '---> The best POPs are: ', strtrim(name_slot[*]+ namefix , 1) 
   endelse
Endif


End

