
function tele_limit, DEC=dec0, phi=phi0, lat=lat0

; Purpose   : Given the limt altitude of a telescope and its position, calculate 
;		the available hour angles for all DEC for the telescope.
;
; Input     : DEC, in DEGREE
;	      Phi, the altitude limit of the telescope.   in Degree. Default is 25 degree
; 	      LAT, latitude of the telescope. in DEgree. Default: CHARA lat.
;
; Output    : HR_range, the hour angle range that the telescope can observe for given DEC.
;
; Usage     : hr_range = tele_limit(dec=findgen(181)-90. [, phi= 25d, lat=30d ] )
;
;
; NOTE      : This routine does NOT work well for Declination higher than 90 -lat, so I estimate those
;		values to be 24 hrs, which is the same as at phi0=0 degree and will not affect the available ;               hour angle for observing
;
;	      Not double checked for negative latitude yet!
;
; MZ, 08/12/05

If n_elements(phi0) eq 0 then phi0 = 25d              ; Default for CHARA
if n_elements(lat0) eq 0 then lat0 = 34.22438d        

dec = dec0/180d *!dpi  	; convert to radian
phi   = phi0 / 180d *!dpi
lat   = lat0 / 180d *!dpi

tmp = -sin(dec)/cos(lat)
idx00= where(abs(tmp) gt 1d)
theta = asin(-sin(dec)/cos(lat))		;  theta = 0   --> west
;                                                  theta = 180 --> east


r = 1d 		; radius of the big circle.

r1 = sqrt( r^2 - (r*sin(theta) *cos(lat))^2)

h = r *sin(phi)
L = h /cos(lat)

height = L + r*sin(theta)*sin(lat)
hr = acos(height/r1)/ !dpi * 24d                ; Hour angle in Hours.


tmp = height/r1
idx01= where(abs(tmp) gt 1d)
if idx01[0] ne -1 then hr(idx01) =0d					; make the values below limit zero
if idx00[0] ne -1 then hr(idx00) =0d

If lat ge 0 then begin
	idx1 =  where( dec0 ge (90.- (lat0 - phi0)))
		if idx1[0] ne -1  then  hr[idx1] = 24d
	idx2 = where(dec0 le ((lat0-phi0) -90.))
		if idx2[0] ne -1  then 	hr[idx2] = 0d
        idx3 = where(dec0 gt 90.-lat0)
		if idx3[0] ne -1 then hr[idx3] = 24d                 ; ROUGHLY estimate the values above dec= 90- lat0 to be 24!!!
endif else if lat lt 0 then begin 
        idx1 =  where( dec0 ge (90.- (lat0 - phi0)))
                if idx1[0] ne -1 then  hr[idx1] = 0d
        idx2 = where(dec0 le ((lat0-phi0) -90.))
                if idx2[0] ne -1  then   hr[idx2] = 24d
	idx3 = where(hr(where(dec0 gt -90.+lat0)) eq !values.D_NAN )    ; Need to check for this idx3 and idx4
                if idx3[0] ne -1 then hr[idx3] = 24d
        idx4 = where(hr(where(dec0 gt 0)) eq !values.D_NAN )
                if idx4[0] ne -1 then hr[idx3] = 0d


endif

hr = hr/2. 
hr_range = dblarr(2, n_elements(hr))
hr_range[0,*] = 0. - hr
hr_range[1, *]= 0. + hr

;stop
return, hr_range

END
