
;PRO MIRCLOG

Pro mirclog, dir=dir, date=date, badfile=badfile, verbose=verbose

if (keyword_set(dir) eq 0) and (keyword_set(date) eq 0) then begin
	print,'Syntax - mirclog, dir=dir, date=date [, badfile=badfile, /verbose]'
	print,'         "dir" does not include the subdirectory of "date"'
        print,"         e.g., dir = '/n/Bahamut2/monnier/DATA/2007May'"
        print,"               date ='2007May07'"
        print,"               badfile=['mirc0000.fits', 'mirc0012.fits'] (if applicable)"
	return
endif

If n_elements(dir) eq 0 then dir=''

dirlen = strlen(dir)
If strmid(dir, dirlen-1,1) ne '/' then dir=dir+'/'

dir2  = dir+date
fname = file_search(dir2,'*', count=cnt) 
namelen = strlen(fname[0])
format1 = '(A12, t22, A7, 1x, I3, 5x, A4, 2x,A4,3x, A4, 3x, 2(A2, 1x))'

IF cnt ge 1 then begin
	nfile = n_elements(fname)
	blocknum = 0
	endflag = 0
        typechange = 1
        type0 = ''     
	mode0 = ''
        count = 0
        source0 =''
        startnum= '0000'
	shutter0 =['', '']
	renewshut= 1 
	multi_shut = 0    ; indicate whether there are two or more continous shutter sequence or not.

        openw, lun, date+'.mirclog',/get_lun

       	printf,lun, '## MIRC Log for '+date
	printf,lun, '## Automatically created by mirclog.pro on '+ systime()
	printf,lun, '## Hand-Edited by ______ on ______'
        printf,lun, '##' 
       	printf,lun, 'DIR   '+strmid(fname[0], 0,namelen-13)
       	printf,lun, '##Source             MODE     Block  Type  Start  End    SHUTBLOCKS'

	FOR n=0, nfile-1 do begin
                if keyword_set(verbose) then print, '--> mirclog:  Reading '+fname[n]
                if n_elements(badfile) ne 0 then begin

                    badind=where(dir2+'/'+badfile eq fname[n], cnt)
                    if cnt ge 1 then begin 
                           print, '--> mirclog: Skipping badfile: '+fname[n]
                           continue
                    endif
                endif

		fits_read, fname[n], data, head,/header_only

		if n_elements(head) lt 140 then begin 
			print, '--> Do not have enough HEADER info. Skip to next file!!!'
			continue
		endif
		source = strmid(head[22],11,12)
		tmp    = strpos(source, "'")
		If tmp ne -1 then source = strmid(head[22],11,tmp)
		source = source + '         '               

        	mode   = strmid(head[140],11,10)
        	type   = strtrim(strmid(head[137],11,6),2)
	        	If type eq 'BACKGR' then type =  'BG'
	        	If type eq 'FOREGR' then type =  'FG'
			If type eq 'BEAM1'  then type =  'B1'
			If type eq 'BEAM2'  then type =  'B2'
			If type eq 'BEAM3'  then type =  'B3'
                        If type eq 'BEAM4'  then type =  'B4'
                        If type eq 'BEAM5'  then type =  'B5'
                        If type eq 'BEAM6'  then type =  'B6'


                IF (type0 ne type) or (source0 ne source) then begin 

			If ((type0 eq 'DATA') or (type eq 'DATA')) or (source0 ne source)  and (n ne 0) then  blocknum++
			endnum   = strtrim(string(fix(startnum)+count),2)

			If source0 ne source then changesrc = 1 else changesrc = 0
			If (changesrc eq 1) and (n ne 0 ) then begin
                                if n_elements(shutter) eq 0 then shutter = shutter0 
				If type0 eq 'DATA' then $
					printf,lun,  source0, mode0, blocknum-1, type0, startnum, endnum,  shutter[0], format=format1 $
				else if type0 ne 'DATA'	then $
					printf,lun,  source0, mode0, blocknum-1, type0, startnum, endnum, format=format1
				printf,lun, '##'
				renewshut = 1
				shutter = shutter0
			Endif
                        If source0 eq source then begin
 				If (type0 eq 'DATA') and (type ne 'DATA') then begin 
			      		If renewshut  eq 1 then begin 
						shutter = shutter0
						shutter[0] = strtrim(string(blocknum),2) 
						renewshut = 0
                                        endif else begin  
						shutter[1] = strtrim(string(blocknum),2)
						renewshut =1
					endelse
                              		printf,lun,  source0, mode0, blocknum-1, type0, startnum, endnum,  shutter, format=format1 
				endif Else $
				if (type0 ne 'DATA') and (type eq 'DATA') and (blocknum ne 0) then begin
                                                if n_elements(shutter) eq 0 then shutter=shutter0
						shutter[0] = strtrim(string(blocknum-1),2) 
                                                renewshut = 0
                        		printf,lun,  source0, mode0, blocknum-1, type0, startnum, endnum,  format=format1
        	        	endif Else $
				If (type0 ne 'DATA') and (type ne 'DATA') and (n ne 0) then $
					printf, lun, source0, mode0, blocknum, type0, startnum, endnum, format=format1
				
			Endif

			startnum = strtrim(string(fix(strmid(fname[n], namelen-9,4))),2)
			mode0    = mode 
			count = 0 

		Endif  Else if (type0 eq type) and (source eq source0) then count++
              
		If n eq nfile-1 then begin 
			endnum   = strtrim(string(fix(startnum)+count),2)
                        if n_elements(shutter) eq 0 then shutter=shutter0

			If type eq 'DATA' then $                        
			printf, lun, source0, mode0, blocknum, type0, startnum, endnum, shutter[0], format=format1 $
			else printf, lun, source0, mode0, blocknum, type0, startnum, endnum,  format=format1
		Endif

                source0 = source
		type0   = type
        Endfor

	free_lun, lun
        Print, '--> Generated log file: '+date+'.mirclog  for the data of '+date+'.'

Endif else begin
Print, '--> No files found in: '+ dir2
print, '--> Returning...'
endelse

end
