; JDM 03Dec00
;
; IOTA3 Project -- Planning and Helper Utilities
;
; calc_uv
;
; This program calculates the (u,v) for a given set of parameters, also DELAY
;
; USAGE
;
pro calc_uv,u=u,v=v,w=w,ha=ha,baseline_vector=baseline_vector,$
            declination=declination, latitude = latitude, zenith=za

;  OUTPUT:
;	u,v,w:	Returns u,v,w in same units as baseline_vector
;		dimensions n, where n =n_elements(ha)

;  INPUT:
;       HA: hour angle  in hours (0=transit) 
;           dimensions n -- can be vector or single #.. this procedure will
;           return the u,v,w for each entry in this vector.
;    baseline_vector:  This is the baseline vector.  Units are 
;			(East, North, Up) -- as shown in IOTA Manual
;			Use whatever units you want (wavelengths..)
; ** SHOULD CHANGE THIS TO MATCH IOTA CONVENTION!
;    declination: Declination of the source in DEGREES
;    latitude   : Latitude of observatory DEGREES
;		  DEFAULT: IOTA: 31 deg 41' 29.58" = 31.691550

; REFERENCE:
; I used the formulas on the AIPS++ glossary page:
;  http://aips2.nrao.edu/docs/glossary

if (n_elements(ha) eq 0) then begin
  print,'USAGE:'
  print,'calc_uv,u=u,v=v,w=w,ha=ha,$
  print,'        baseline_vector=baseline_vector,$'
  print,'        declination=declination, latitude = latitude
  print,' NOTE: baselines vector (East, North,  up)
  return
endif

if (n_elements(latitude) eq 0) then latitude = 31.691550 ; IOTA
b_east=baseline_vector(0)
b_north=baseline_vector(1)
b_up=baseline_vector(2)

; Convert hour angle from HOURS to radians.
ha_rads = ha*!dpi/12.d0

; Convert Declination from degrees to radians.
dec_rads=declination*!dpi/180.d0

; Convert Latitude from degrees to radians
lat_rads=latitude*!dpi/180.d0

; NOw convert from (east,north,up) to (x,y,z) as defined by AIPS++
;    x = to HA 0, dec 0
;    y = to ha -6, dec 0 (east)
;    z= dec 90( north celestial)

bx=-sin(lat_rads)*b_north + cos(lat_rads)*b_up
by=b_east
bz=cos(lat_rads)*b_north + sin(lat_rads)*b_up

cosza=sin(lat_rads)*sin(dec_rads)+cos(lat_rads)*cos(dec_rads)*cos(ha_rads)
za=acos(cosza < 1.0)*180.d0/!dpi
; Now convert bx,by,bz to (u,v,w)
u=sin(ha_rads)*bx + cos(ha_rads)*by  ; Independent of DEC
v=-sin(dec_rads)*cos(ha_rads)*bx + sin(dec_rads)*sin(ha_rads)*by +  $
   cos(dec_rads)*bz
w=cos(dec_rads)*cos(ha_rads)*bx - cos(dec_rads)*sin(ha_rads)*by +   $
   sin(dec_rads)*bz

; Done.

end
    
 


		

        
       
