function dir_path, direct_array, dir_delim
;+
; NAME:
;       dir_path
; PURPOSE:
;       Construct and return a string specifying a directory
;       from a generic array of sub-directories which determine a path
;       (thus works on both Unix and VMS operating systems).
; CALLING:
;       directory = dir_path( direct_array, dir_delim )
; INPUT: 
;       direct_array = string array giving a path of sub-directories, which
;                     upon concatenation specifies the subdirectory.
;                                               (default is current directory).
;                     Do not include the operating system specific syntax for
;                     directories, that is, leave out the "/" or ".",
;                     they are automatically added depending on op.sys.
;                     Leading & trailing blanks are trimmed, default ="".
; OUTPUT (optional):
;       dir_delim = string used by op.sys. to delimit (end) the directory spec.
;                     (e.g. "]" on VMS, otherwise "/" on UNIX).
;
;       Function returns string specifying the directory on current op.sys.
; HISTORY:
;       Written, Frank Varosi NASA/GSFC 1989.
;-
	Ndir = N_elements( direct_array )
	if (Ndir LE 0) then return,""
	dir_array = strtrim( direct_array, 2 )

	w = where( [strlen( dir_array )] GT 0, Ndir )
	if (Ndir LE 0) then return,""

	if ( !VERSION.OS  EQ "vms" ) then begin

		dir_delim = "]"
		directory = dir_array(w(0))
		for i=1,Ndir-1 do directory = directory + "." + dir_array(w(i))
		directory = "[." + directory + "]"

	  endif else begin

		dir_delim = "/"
		directory = dir_array(w(0)) + "/"
		for i=1,Ndir-1 do directory = directory + dir_array(w(i)) + "/"
	   endelse

return, directory
end
