; 2007Aug21 JDM	Align DAQDATA with header times... as best as possible.
; 2008Mar03 JDM strange bug for idl 7.0?
;
;
pro mirc_aligndaq,datadir,daqdir,daqdata

;goto,skipload
;datadir='/n/Bahamut2/monnier/DATA/MIRC_COADD/2007Jul02/'
;daqdir='/n/Bahamut2/monnier/DATA/MIRC_DAQData/2007Jul02/' 



if keyword_set(daqrate) eq 0 then daqrate=1000. ;hz

files_data=file_search(datadir+"/*.fits")
if (files_data(0) eq '') then begin
 print,' no FITS files in '+datadir
 return
endif

files_daq =file_search(daqdir+"/mircdaq*")
if (files_daq(0) eq '') then begin
 print,' no mircdaq files in '+daqdir
 return
endif

;------------------
; First read ALL HEADERS [this should be done in an earlier phase of
; data reduction since this is very repetitive on disk access....
;
nfiles=n_elements(files_data)
mjd = dblarr(nfiles)

for i=0,nfiles-1 do begin
  head=headfits(files_data(i))
  ft=sxpar(head,'DATE') ; time from fits file -- computer time
    date = strmid(ft,0,10)
    year = strmid(ft,0,4)
    mon  = strmid(ft,5,2)
    day  = strmid(ft,8,2)
    hr   = strmid(ft,11,2)
    min  = strmid(ft,14,2)
    sec  = strmid(ft,17,2)
    time = hr*3600.d0+min*60.d0+sec*1d0
    jdcnv,year,mon,day,time/3600.d0,jd
    mjd(i) = jd- 2400000.5d0
endfor

ndaq=n_elements(files_daq)

;sort files
files_daq = files_daq(sort(files_daq))
firstone=0
for i=0,ndaq-1 do begin
daqdata0=loadmircdaq(files_daq(i))
info=size(daqdata0,/type)


if info eq 8 then begin

; Filter out most of the daqdata where data is not being taken....
volttemp = daqdata0.volt0
voltsbin = (volttemp)/5.0
inup = where(voltsbin gt 0.5,ct)
if ct gt 0 then voltsbin(inup)=1
indown = where(voltsbin le 0.5,ct)
if ct gt 0 then voltsbin(indown)=0
voltsbin=smooth(voltsbin,5,/edge)

in=where(voltsbin gt 0,ctgood)

if (ctgood gt 0) then begin
  firstone=firstone+1
    if firstone eq 1 then daqdata=daqdata0(in) else daqdata=[daqdata,daqdata0(in)]
endif
endif

print,'Accumulating MIRC DAQ info: ',files_daq(i),i,ndaq
endfor
;stop

skipload:

data_sec = mjd*3600d0*24d0

jdcnv,1970.,1,1,0.d0,jd0_daq
mjd_daq0 = jd0_daq-2400000.5d0
daq_sec = mjd_daq0*3600d0*24d0+1d0*daqdata.seconds + daqdata.nanoseconds/1d9

;note:  we expect either a 7 or 8 hour offset in the time, since the computer
;taking data is on UT and the clock on the webdaq is local time.
; data - daq = 28800 (8 hrs) or 25200 (7 hours)

; find indices when file saving STARTS...
sz=n_elements(daqdata)
din0=daqdata.din0
din1=daqdata.din1
din2=daqdata.din2
din3=daqdata.din3
volt0=daqdata.volt0


;voltsbin = (volt0-min(volt0))
;voltsbin = voltsbin/max(voltsbin)
; The top two lines failed on 2007Aug11 due to a bad value... max 1e10 volts !!
; so will just assume TTL
voltsbin = (volt0-0.0)
voltsbin = voltsbin/5.0 

inup = where(voltsbin gt 0.5,ct)
if ct gt 0 then voltsbin(inup)=1
indown = where(voltsbin le 0.5,ct)
if ct gt 0 then voltsbin(indown)=0

datablocks=(voltsbin-shift(voltsbin,-1))(0:(sz-2))
dataon = where(datablocks lt -.5,cton)
dataoff =where(datablocks gt .5,ctoff) ; add one because of timing kludge.

; if data taking was ongoing while daq was turned on then
; there may be a problem that dataon<dataoff. check this and fix if necessary
if (dataon(0) gt dataoff(0)) then begin
  dataoff=shift(dataoff,-1)
  ; check ending
  datadiff=dataoff-dataon
  inbad = where(datadiff lt 0,ctbadyes)
  if ctbadyes gt 0 then begin
     dataoff(inbad) = dataon(inbad) + median(datadiff)
  endif

endif

; most probably offset between the two clocks by finding
;all the differences between dataon times and mjds.

; if clocks are reasonably close to right, then the optimal offset should
; be between 6 and 9 hours [either 7 or 8 really. but this allows for
; major clock drifts]

mintrial=6.5*3600.
maxtrial=8.5*3600. ; seconds

trial_offsets = pvector([mintrial,maxtrial,(maxtrial-mintrial+1)])
ntrials=n_elements(trial_offsets)

chi2=fltarr(ntrials)
maxerr=fltarr(ntrials)
numclose = fltarr(ntrials)
  closetime = 1.5; seconds


for i=0l,ntrials-1l do begin
 for j=0l,cton -1l do begin
   daqtime0=daq_sec(dataon(j))+trial_offsets(i)
   mindiff = min(abs(data_sec - daqtime0))
   chi2(i)=chi2(i)+ mindiff*mindiff
   maxerr(i)= maxerr(i) > mindiff
   if mindiff le closetime then numclose(i)=numclose(i)+1.
 endfor
;print,i,ntrials,trial_offsets(i),maxerr(i)
endfor
inchi2=where(chi2 eq min(chi2))
inminerr=where(maxerr eq min(maxerr))
inclose = where(numclose eq max(numclose),ctclose)

if ctclose gt 1 then begin
  in0=where(chi2(inclose) eq min(chi2(inclose)))
  inclose=inclose(in0)
endif

print, 'Using offset (hrs): ',trial_offsets(inclose)/3600.
print, ' Num of data pulses with a file within ',closetime,' secs: '
print,'     ',numclose(inclose(0)),' out of ',n_elements(dataon)
print,  '  MAX timing alignment errors (sec): ',maxerr(inclose)

if min(maxerr) gt 1.5 then begin
  print,' there seems to be some inconsistency between the
  print,' recorded syncronization of files and the actual files...  '
  stop
endif
;stop

; now line up the dataon pulses with the corresonding files.
dataon_files = strarr(cton)
new_daqtime = daq_sec(dataon) + trial_offsets(inclose(0))
mindiffs=fltarr(cton)
for i=0,cton-1 do begin
   diff = new_daqtime(i)-data_sec
   in=where( abs(diff) eq min(abs(diff)))
   mindiffs(i)=diff(in(0))
;   if (abs(diff(in)) gt min(maxerr)) then begin
;     print,' some error'
;     stop
;   endif
   if min(abs(diff)) le closetime then dataon_files(i) = files_data(in(0)) $
    else begin
      dataon_files(i) = ''
    endelse
print,diff(in(0)),'Lining Up: ',dataon(i),dataon_files(i)
endfor 

; Pack up structure with just what we need and save a copy...

daqdata0={file: ' ', num: fix(100), din0:ptr_new([0]),din1:ptr_new([0]),$
    din2:ptr_new([0]),din3:ptr_new([0]) }

daqdata=replicate(daqdata0,cton)

for i=0,cton-1 do begin
   daqdata(i).file=dataon_files(i)

   if dataon_files(i) ne '' then begin
     q=strpos(dataon_files(i),'.fits')
     daqdata(i).num=fix(strmid(dataon_files(i),q-4,4))
   endif else daqdata(i).num = -1

   daqdata(i).din0 = ptr_new(din0(dataon(i):dataoff(i)))
   daqdata(i).din1 = ptr_new(din1(dataon(i):dataoff(i)))
   daqdata(i).din2 = ptr_new(din2(dataon(i):dataoff(i)))
   daqdata(i).din3 = ptr_new(din3(dataon(i):dataoff(i)))

endfor

; find files that don't have sync and printout.
for i=0,nfiles -1 do begin
  in=where(daqdata.file eq files_data(i),ct)
  if ct eq 0 then print,' NO DAQINFO for ',files_data(i)
  if ct gt 1 then print,' TOO MUCH DAQINFO for ',files_data(i)
endfor

;stop
;------------------

end
